
/********************
  pin   12  ds1820
  pin   11  ds1820
  pin 13 red  outPin = 13,выходного сигнала (для управления симистором)
  pin   3  на 4-х разъем
  pin   2  детектор 0
  Пин Пищалки А2

  pin A1 черный  Чтение RMS A1

  encA A4  encoder
  encB A3
  encBtn A5
***/

static uint8_t inPin = 2;  // Пин сигнала от детектора нуля (используется внешнее прерывание INT0)
//желтый


#include <menu.h>
#include <menuIO/liquidCrystalOut.h>
#include <menuIO/serialOut.h>
#include <menuIO/serialIn.h>
#include <menuIO/encoderIn.h>
#include <menuIO/keyIn.h>
#include <menuIO/chainStream.h>
#include <Wire.h>
#include <OneWire.h>
using namespace Menu;
LiquidCrystal lcd(8, 9, 4, 5, 6, 7);

encoderIn<A4, A3> encoder; //simple quad encoder driver
encoderInStream<A4, A3> encStream(encoder, 4); // simple quad encoder fake Stream

//a keyboard with only one key as the encoder button
keyMap encBtn_map[] = {{ -A5, defaultNavCodes[enterCmd].ch}}; //negative pin numbers use internal pull-up, this is on when low
keyIn<1> encButton(encBtn_map);//1 is the number of keys

menuIn* inputsList[] = {&encStream, &encButton};
chainStream<2> in(inputsList);//2 is the number of inputs

volatile int cntr = 0; // счетчик в обработчике прерывания
volatile unsigned long Usumm = 0; // переменная для хранения суммы квадратов

int ust_U = 0;
int angle_U = 2550 ; // подбирать под конкретный проц
byte zero = 0;

word tmp;

OneWire dss[] = { OneWire(11), OneWire(12) };   // bus on pin 0 and 1
volatile int cels[2];  //Массив для хранения температуры 0 бак 1 колона
byte index = 0; //номер датчика температуры
boolean modeRect = 0; //режим работы НБК или ректификация
int TempTankRcf,  TempColumnRcf;

TOGGLE(modeRect, setMode, "Mode: ", doNothing, noEvent, noStyle //,doExit,enterEvent,noStyle
       , VALUE("Rect", HIGH, doNothing, noEvent)
       , VALUE("Dis", LOW, doNothing, noEvent)
      );

MENU(subMenuRect, "Set alarm", doNothing, anyEvent, noStyle
     , FIELD(TempTankRcf  , "Tank  ", " C", 200, 1000, 10, 0, doNothing, noEvent, noStyle)
     , FIELD(TempColumnRcf, "Column", " C", 500, 1000, 10, 2, doNothing, noEvent, noStyle)
     , EXIT("<<")
    );

MENU(onLine, "onLine" , doNothing,  noEvent, noStyle
     , FIELD(cels[0] , "Tank  ", " C", 0, 0, 0, 0, doNothing, noEvent, noStyle)
     , FIELD(cels[1] , "Column", " C", 0, 0, 0, 0, doNothing, noEvent, noStyle)
     , EXIT("<<")
    );

MENU(mainMenu, "Main menu", doNothing, noEvent,  noStyle
     , FIELD( ust_U, "U =", " V", 0, 230, 10, 1, doNothing, anyEvent, noStyle)
     , SUBMENU(setMode)
     , SUBMENU(subMenuRect)
     , SUBMENU(onLine)
    );

#define MAX_DEPTH 2

MENU_OUTPUTS(out, 2
             , LIQUIDCRYSTAL_OUT(lcd, {0, 0, 16, 2})
             , NONE
            );
NAVROOT(nav, mainMenu, MAX_DEPTH, in, out); //the navigation root object

void setup() {

  //------ Timer1 ---------- Таймер задержки времени открытия триака после детектирования нуля (0 триак не откроется)

  TCCR1A = 0x00;  //
  TCCR1B = 0x00;    //
  TCCR1B = (1 << ICNC1) | (0 << CS12) | (1 << CS11) | (1 << CS10); // Тактирование от CLK.
  OCR1A = 2300;           // Верхняя граница счета. Диапазон от 0 до 65535.
  TIMSK1 |= (1 << OCIE1A); // Разрешить прерывание по совпадению

  attachInterrupt(0, zero_crosss_int, RISING);//вызов прерывания при детектировании нуля


  // Инициализация АЦП
  ADMUX = (0 << REFS1) | (1 << REFS0) | (0 << MUX2) | (0 << MUX1) | (1 << MUX0); // A1
  ADCSRA = B11101111;

  // Encoder
  DDRC = (0 << DDC5);  //A5 на вход, подтяжка+5v
  PORTC = (1 << PORTC5);

  // Пин выходного сигнала (для управления симистором) Port B5 PIN 13
  DDRB |= B00100000;
  PORTB = (0 << PORTB5);

  // Пин сигнала от детектора нуля  PIN 2
  DDRD &= B11111011; //  pinMode(2, INPUT);

  // Пин Пищалки А2
  DDRC |= B00000100; //  pinMode(A2, OUTPUT);

  Serial.begin(115200);
  encoder.begin();
  lcd.begin(16, 2);
  nav.showTitle = false;
}

void loop() {
  unsigned long previousMillis = 0;
  unsigned long previousMillis_info = 0;
  unsigned int real_U = 0; //переменная расчета RMS напряжения
  static boolean n = 0; // флаг работы: запрос температуры или её чтение
  byte step_ = 0;
  while (1) {

    if (zero == 8) {//расчет среднеквадратичного//zero - количество полупериодов дря рассчета среднеквадратичного
      float  sqrtUsum = Usumm / cntr; //
      real_U = sqrt(sqrtUsum); //
      cntr = 0;
      Usumm = 0;// обнуляем суммы напряжений
      zero = 0;
    }

    //================================================================

    if (ust_U == 0) {
      TIMSK1 = 0x00;
      PORTB &=  ~(1 << PORTB5); //Выключаем тиристор
      angle_U = 2550;
    }
    else {
      TIMSK1 |= (1 << OCIE1A) | (1 << TOIE1);
    }

    step_ = abs (real_U - ust_U);

    if (step_ <= 1) {
      step_ = 1;
    }
    if (step_ >= 20) {
      step_ = 20;
    }

    //=====================================Стабилизатор напряжения
    if (millis() - previousMillis  >= 120) { //раз в пол секунды регулируем, индицируем
      previousMillis  = millis();

      if (real_U > ust_U + 1) {
        angle_U = angle_U + step_;
      }
      if ((real_U + 1) < ust_U ) {
        angle_U = angle_U - step_;
      }

      if (angle_U > 2550) {
        angle_U = 2550;
      }
      if (angle_U <= 1) {
        angle_U = 1;
      }
      OCR1A = angle_U;
    }

    nav.poll();

    //=========================Информация в порт раз в секунду

    if (millis() - previousMillis_info >= 1000) {
      previousMillis_info = millis();


      if (n = !n) rqTemp(dss[index]); // запрос на конвертацию
      else   {
        reedTemp(dss[index]);  // или чтения температуры
        index++;
      }
      if (index == 2) index = 0;

      PORTC &= B11111011; //LOW выключаем пищалку PIN A2 порт С3
      //разбор полетов по температуре, если превышает, начинаем пищать
      if ((cels[0] > TempTankRcf  || cels[1] > TempColumnRcf) && (modeRect == 1))
      {
        if (n == 1) PORTC |= B00000100; // digitalWrite(A2, HIGH);

      }


      Serial.print("real_U "); //Измеренное среднеквадратичное напряжение
      Serial.println(real_U);

      Serial.print("ustU  "); //Установленное выходное напряжение
      Serial.println(ust_U);

      Serial.print("ang "); //Установленное выходное напряжение
      Serial.println(angle_U);
      Serial.println("--");
    }
  }
}


void rqTemp(OneWire dss) {
  dss.reset();
  dss.write(0xCC, 1);  // skip ROM
  dss.write(0x44, 1);      // start conversion, with parasite power on at the end

}
void reedTemp(OneWire dss) {
  dss.reset();
  dss.write(0xCC, 1);  // skip ROM
  dss.write(0xBE);     // Command read
  cels[index] =  (dss.read() | (dss.read() << 8)) * 5 / 8; //прочитаны 2 байта
}


//======================================================== Прерывание по нулю
void zero_crosss_int() {
  PORTB &=  ~(1 << PORTB5);
  TCNT1 = 0;
  zero++;
}

//======================================================== Обработка таймера по совпадению нуля
ISR (TIMER1_COMPA_vect)
{
  PORTB |=  (1 << PORTB5); //ВЫКЛЮЧИТЬ ТИРИСТОР
}
//======================================================== Обработка прерывания АЦП для расчета среднеквадратичного напр
ISR(ADC_vect)
{
  register uint32_t Uism = (ADCL | ADCH << 8);
  Uism *= Uism;// возводим значение в квадрат
  Usumm += Uism; // складываем квадраты измерений
  cntr++;
}

